package org.sinev.igor.ps.operatordescription;

import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.LinkedHashMap;
import java.util.List;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import org.sinev.igor.ps.elementtype.ElementType;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.xml.sax.SAXException;

/**
 * Manages descriptions of all operators.
 * 
 * @author Igor Sinev 
 */
public class OperatorNamingMap {
	
	private LinkedHashMap<String, OperatorDescription> myMap;
		
	/**
	 * Creates an empty OperatorNamingMap. 
	 */
	public OperatorNamingMap() {
		myMap = new LinkedHashMap<String, OperatorDescription>();
	}
	
	/**
	 * Creates a new OperatorDescription and adds it to the naming map.
	 * 
	 * @param n name of operator
	 * @param a contains argument types of the operator
	 * @param r contains types of the operator return values
	 */
	public void addDescription(String n, ElementType[] a, ElementType[] r) {	
		myMap.put(n, new OperatorDescription(n, a, r));
	}
	
	/**
	 * Creates a new OperatorDescription and adds it to the naming map.
	 * 
	 * @param n name of operator
	 * @param a contains argument types of the operator
	 * @param r contains types of the operator return values
	 * @param g string containing list of operator groups
	 */
	public void addDescription(String n, ElementType[] a, ElementType[] r, String g, String d) {	
		myMap.put(n, new OperatorDescription(n, a, r, g, d));
	}
	
	/**
	 * Looks for an operator with specified name 
	 * in the map.
	 * 
	 * @param s operator name
	 * 
	 * @return description or null if no such operator found
	 */
	public OperatorDescription getDescription(String s) {
		OperatorDescription d = myMap.get(s);
		return (d == null) ? OperatorDescription.undefinedOperator : d;		
	}
	
	/**
	 * Obtains the list of all stored operators.
	 * 
	 * @return list of names of operators.
	 */
	public List<String> getNames() {
		List<String> r = new ArrayList<String>(myMap.values().size());
		for (OperatorDescription d : myMap.values())
			r.add(d.getName());
		return r;
	}
	
	/**
	 * Reads a map from XML file.
	 * 
	 * @param file to get map from
	 * 
	 * @return read map
	 * 
	 * @throws ParserConfigurationException
	 * @throws SAXException
	 * @throws IOException
	 */
	public static OperatorNamingMap getFromXML(File file) throws ParserConfigurationException, SAXException, IOException {
		Document document;

        DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
        factory.setNamespaceAware(true);
        DocumentBuilder builder = factory.newDocumentBuilder();
          
        document = builder.parse(file);
            
        OperatorNamingMap m = new OperatorNamingMap();
        Element root = document.getDocumentElement();
        
        NodeList operators = root.getElementsByTagName("operator");
        for (int i = 0; i < operators.getLength(); i++) {
        	Node operator = operators.item(i);
        	String name = operator.getAttributes().getNamedItem("name").getNodeValue();
        	ArrayList<ElementType> arguments = new ArrayList<ElementType>();
        	ArrayList<ElementType> returns = new ArrayList<ElementType>();
        	String group = "";
        	String comment = "";
        	NodeList nodes = operator.getChildNodes();
        	for (int j = 0; j < nodes.getLength(); j++) {
        		Node n = nodes.item(j);
        		String nodeType = n.getNodeName();
        		if (nodeType == "group") {
        			group = n.getAttributes().getNamedItem("name").getNodeValue();
        		}
        		else if (nodeType == "comment") {
        			comment = n.getFirstChild().getNodeValue().trim();
        		} 
        		else if (nodeType == "argument") {
        			arguments.add(ElementType.getByName(n.getAttributes().getNamedItem("type").getNodeValue()));        			
        		}
        		else if (nodeType == "return-element") {
        			returns.add(ElementType.getByName(n.getAttributes().getNamedItem("type").getNodeValue()));        			
        		}
        	}
        	m.addDescription(
        			name, 
        			arguments.toArray(new ElementType[arguments.size()]), 
        			returns.toArray(new ElementType[returns.size()]), 
        			group, 
        			comment
        		);
        }
        return m;
	}

}
